'use strict';

const _ = require('lodash');
const BaseTemplate = require('../../cloudformation/main/base');
const Lambda = require('./lambda');

module.exports = (functions, props) => {
  const lambda = new Lambda(props);

  const addServiceTemplate = (functionObject) => {
    const service = functionObject.serviceIdentifier;

    if (!_.has(props.cloudFormation.stacks, service)) {
      const baseTemplate = new BaseTemplate();
      props.cloudFormation.stacks[service] = baseTemplate.compile(
        `${service} stack`
      );

      props.cloudFormation.stacks.mainStack.Resources[service] = {
        Type       : 'AWS::CloudFormation::Stack',
        Properties : {
          TemplateURL      : `https://s3.amazonaws.com/${props.cloudFormation.outputs.SqueezerDeploymentBucket}/` +
          `cloudformation/${service}-template.json`,
          TimeoutInMinutes : 10,
          Parameters       : {
            ApiGatewayRestApiId    : {
              'Fn::GetAtt' : ['ApiGatewayStack', 'Outputs.ApiGatewayRestApiId']
            },
            IamRoleLambdaExecution : {
              'Fn::GetAtt' : ['iamStack', 'Outputs.IamRoleLambdaExecution']
            }
          }
        },
        DependsOn  : ['iamStack', 'ApiGatewayStack']
      };
    }
  };

  _.forEach(functions, (functionObject) => {
    addServiceTemplate(functionObject);
    lambda.compile(functionObject);
  });
};
